//
// The analog read example replies with the current value of first analog input.
//
// The Python client example is:
//
// import yami
// agent = yami.Agent()
// def readValue():
//     msg = agent.send("udp://192.168.1.177:12345", "ADC", "read");
//     msg.wait_for_completion()
//     reply = msg.get_reply()
//     del msg
//     return reply["value"]
//
// print readValue()
//

// Note: these headers have to be included in this order:
#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <YAMI4.h>

// set up your MAC and IP address:
byte mac[] = {  
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 1, 177);

// local port to listen on:
unsigned int localPort = 12345;

EthernetUDP udp;
YAMI4Agent agent;

// this function will receive all YAMI4 messages:
void call(IPAddress remoteIp, uint16_t remotePort,
    enum yami_message_type message_type,
    const char object_name[], const char message_name[], const char exception_msg[],
    int64_t message_id,
    const uint8_t body[], size_t body_size)
{
  // ignore the message header and body,
  // always reply with the current A0 value
  
  const int32_t sensor_value = analogRead(A0);
  
  const int32_t num_of_entries = 1;
  
  const size_t reply_buffer_size = 100u;
  uint8_t reply_body[reply_buffer_size]; // serialized reply body
  size_t reply_body_size = 0u; // actual size of serialized reply

  // number of fields
  yami_put_integer(reply_body, reply_buffer_size, &reply_body_size, num_of_entries);

  // field name
  yami_put_cstring(reply_body, reply_buffer_size, &reply_body_size, "value");

  // field type
  yami_put_type(reply_body, reply_buffer_size, &reply_body_size, yami_integer);
  
  // field value
  yami_put_integer(reply_body, reply_buffer_size, &reply_body_size, sensor_value);
  
  // send the reply back to client
  agent.sendReply(remoteIp, remotePort, message_id, reply_body, reply_body_size);
}

void setup()
{
  // start the Ethernet and UDP:
  Ethernet.begin(mac,ip);
  udp.begin(localPort);

  // initialize YAMI4 agent:
  agent.init(udp, &call);
}

void loop()
{
  // if there is any incoming message,
  // the call function will be executed:
  agent.receive();

  delay(100);
}
