//
// The LED blink example receives an "ON" or "OFF" message
// and blinks the LED accordingly.
//
// The Python client example invocation is just:
//
// import yami
// agent = yami.Agent()
// agent.send_one_way("udp://192.168.1.177:12345", "LED", "ON");
//

// Note: these headers have to be included in this order:
#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <YAMI4.h>

// set up your MAC and IP address:
byte mac[] = {  
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 1, 177);

// local port to listen on:
unsigned int localPort = 12345;

EthernetUDP udp;
YAMI4Agent agent;

const int LED = 2; // use pin 2 to drive the LED

// this function will receive all YAMI4 messages:
void call(IPAddress remoteIp, uint16_t remotePort,
    enum yami_message_type message_type,
    const char object_name[], const char message_name[], const char exception_msg[],
    int64_t message_id,
    const uint8_t body[], size_t body_size)
{
  // ignore the message body, only consider message_name
  
  if (strcmp(message_name, "ON") == 0)
  {
    digitalWrite(LED, HIGH);
  }
  else
  {
    digitalWrite(LED, LOW);
  }
}

void setup()
{
  // start the Ethernet and UDP:
  Ethernet.begin(mac,ip);
  udp.begin(localPort);

  // initialize YAMI4 agent:
  agent.init(udp, &call);

  pinMode(LED, OUTPUT);
}

void loop()
{
  // if there is any incoming message,
  // the call function will be executed:
  agent.receive();

  delay(100);
}
