//
// This example just prints (on the serial line) the text message
// that Arduino receives from remote client.
// It can work with any "print" client example from the YAMI4 package.
//

// Note: these headers have to be included in this order:
#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <YAMI4.h>

// set up your MAC and IP address:
byte mac[] = {  
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 1, 177);

// local port to listen on:
unsigned int localPort = 12345;

EthernetUDP udp;
YAMI4Agent agent;

// this function will receive all YAMI4 messages:
void call(IPAddress remoteIp, uint16_t remotePort,
    enum yami_message_type message_type,
    const char object_name[], const char message_name[], const char exception_msg[],
    int64_t message_id,
    const uint8_t body[], size_t body_size)
{
  // ignore the header, but deserialize the message body
  
  enum yami_result res;
  
  int32_t num_of_entries;
  int32_t len;
  const char content_tag[] = "content";
  const size_t content_tag_len = 7; // length of "content"
  char field_name[8];               // sizeof "content"
  char line[100];
  enum yami_parameter_type type;
  
  // buffer position is updated with each reading operation
  size_t buffer_position = 0u;
  
  // for simplicity this server does not allow unknown fields,
  // so the only expected message format is a regular message
  // with "content" string field
  
  res = yami_get_integer(body, body_size, &buffer_position, &num_of_entries);
  
  if ((res != yami_ok) || (num_of_entries != 1))
  {
    return;
  }
  
  // read length of field name
  
  res = yami_get_integer(body, body_size, &buffer_position, &len);

  if ((res != yami_ok) || (len != content_tag_len))
  {
    return;
  }
  
  // read the field name (expect "content")
  
  res = yami_get_raw_cstring(body, body_size, &buffer_position,
      field_name, (size_t)len);
    
  if ((res != yami_ok) || (strcmp(field_name, content_tag) != 0))
  {
    return;
  }
  
  // read type (expect string)
  
  res = yami_get_type(body, body_size, &buffer_position, &type);
  
  if ((res != yami_ok) || (type != yami_string))
  {
    return;
  }
  
  // read length of field string value
  
  res = yami_get_integer(body, body_size, &buffer_position, &len);
  
  if ((res != yami_ok) || (len >= 1000))
  {
    return;
  }
  
  // read the field string value, and print it
  
  res = yami_get_raw_cstring(body, body_size, &buffer_position,
      line, (size_t)len);

  Serial.println(line);
}

void setup()
{
  // start the Ethernet and UDP:
  Ethernet.begin(mac,ip);
  udp.begin(localPort);

  // initialize YAMI4 agent:
  agent.init(udp, &call);

  Serial.begin(9600);
}

void loop()
{
  // if there is any incoming message,
  // the call function will be executed:
  agent.receive();

  delay(100);
}
